from telegram import Update, InlineKeyboardButton, InlineKeyboardMarkup
from telegram.constants import ParseMode
from telegram.ext import ContextTypes

import html_parser
from config import *
from extensions import get_json, format_rating, format_status


async def start_command(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    await update.message.reply_text(
        f"hello, i'm {html_parser.bold(context.bot.first_name)}, an inline image grabber.\n\n"
        f"to get help, use /help",
        parse_mode=ParseMode.HTML
    )


async def help_command(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    await update.message.reply_text(
        f"{html_parser.bold(f"how to use {context.bot.first_name}")}\n\n"
        f"1. open your message box and type:\n"
        f"{html_parser.code(f"@{context.bot.username} &lt;post ID&gt;")}\n"
        f"2. click on the box that has popped up\n"
        f"3. ???\n"
        f"4. done!",
        parse_mode=ParseMode.HTML
    )


async def about_command(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    reply_markup = None
    if general.sourceurl is not None:
        keyboard = [
            [
                InlineKeyboardButton(f"source code", url=general.sourceurl)
            ]
        ]
        reply_markup = InlineKeyboardMarkup(keyboard)

    await update.message.reply_text(
        f"{html_parser.bold(f"about {context.bot.first_name}")}\n"
        f"{context.bot.first_name} is an inline image grabber written in python that grabs images from Danbooru "
        f"(or other similar services).\n\n"
        f"{html_parser.bold("currently configured instance:")}\n"
        f"{html_parser.italic(app.name)} ({app.hostname})",
        parse_mode=ParseMode.HTML,
        reply_markup=reply_markup
    )


async def info_command(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    try:
        post_id = context.args[0]
        message = await context.bot.send_message(update.effective_chat.id,
                                                 f"{html_parser.bold("Information")}\n"
                                                 f"Fetching...",
                                                 parse_mode=ParseMode.HTML)
        post_data = get_json(f"posts/{post_id}")
        if post_data is None:
            await update.message.reply_text(
                f"{html_parser.bold("Error")}: That record was not found.",
                parse_mode=ParseMode.HTML)
            return
        uploader_data = get_json(f"users/{post_data['uploader_id']}")
        # well, we could check the uploader, but why would we do that?

        await context.bot.edit_message_text(
            f"{html_parser.bold("Information")}\n"
            f"ID: {html_parser.code(post_data['id'])}\n"
            f"Uploader: {html_parser.hyperlink(uploader_data['name'],
                                               f"http://{app.host}/users/{post_data['uploader_id']}")} "
            f"{html_parser.hyperlink("»", f"http://{app.host}/posts?tags=user:{uploader_data['name']}")}\n"
            f"Date: {post_data['created_at']}\n"
            f"Size: {post_data['media_asset']['file_size']} .{post_data['media_asset']['file_ext']} "
            f"({post_data['media_asset']['image_width']}x{post_data['media_asset']['image_height']}) "
            f"{html_parser.hyperlink("»", f"http://{app.host}/media_assets/{post_data['media_asset']['id']}")}\n"
            f"Source: {post_data['source']}\n"
            f"Rating: {format_rating(post_data['rating'])}\n"
            f"Score: {html_parser.hyperlink(post_data['score'],
                                            f"http://{app.host}/post_votes?search[post_id]={post_data['id']}&variant=compact")} "
            f"(+{post_data['up_score']} / -{post_data['down_score']})\n"
            f"Favorites: {html_parser.hyperlink(post_data['fav_count'],
                                                f"http://{app.host}/posts/{post_data['id']}/favorites")}\n"
            f"Status: {format_status(post_data)}",
            update.effective_chat.id, message.message_id,
            parse_mode=ParseMode.HTML)
    except (IndexError, ValueError):
        await update.message.reply_text(
            f"{html_parser.bold("Usage")}: {html_parser.code(f"/info &lt;post ID&gt;")}",
            parse_mode=ParseMode.HTML)
